<?php
/**
 * A PHP5 class for interfacing with the Amazon S3 API
 *
 * @package Storage3
 * @category Storage3
 */

/**
 *
 */
 
require_once 'Crypt/HMAC.php';
require_once 'HTTP/Request.php';
require_once 'Net/URL.php';
require_once 'Net/Socket.php';

/**
 * This class is the abstraction layer through which you provide easy access to the Amazon S3 API
 *
 * Takes ($key, $secret, $url)
 * - [str] $key: Your Amazon Web Services Access Key ID
 * - [str] $secret: Your Amazon Web Services Secret Access Key
 * - [str] $url: OPTIONAL (default: http://s3.amazonaws.com/)
 *
 * Example Usage:
 *
 * <pre>
 *   $s3=new storage3("aslkdfjhalskdfjhals", "sdalkjfhaslkjdfhaslkdhjfaslkjdhflas");
 *
 *   $s3->lsBuckets();
 *   $s3->lsFiles("apokalyptik2");
 *
 *   $s3->mkBucket("apokalyptik2"); 
 *   $s3->mkFile("blah do blah", "apokalyptik2", "test", "binary/octet-stream", time()+300);
 *
 *   $s3->getFile('apokalyptik2', 'test');
 *
 *   $s3->rmFile('apokalyptik2', "test");
 *   $s3->rmBucket('apokalyptik2');
 * </pre> 
 */
class Storage3 {
  /**#@+
   * @ignore 
   */
  var $accessUrl;
  var $accessKeyId;
  var $accessSecret;
 /**#@-*/

  /**
   * Constructor
   *
   * Takes ($key, $secret, $url)
   *
   * - [str] $key: Your Amazon Web Services  "Access Key ID"
   * - [str] $secret: Your Amazon Web Services  "Secret Access Key"
   * - [str] $url: OPTIONAL: defaults: http://s3.amazonaws.com/ 
   *
   * @ignore
   */
  function __construct($key, $secret, $url="http://s3.amazonaws.com/") {
    $this->accessUrl=$url;
    $this->accessKeyId=$key;
    $this->accessSecret=$secret;
  }

  /**
   * Get a list of all your s3 buckets (directories)
   *
   */
  function lsBuckets() {
    $httpDate = gmdate(DATE_RFC2822);
    $tosign="GET\n\n\n{$httpDate}\n/";
    $authSigned="AWS {$this->accessKeyId}:".$this->strSign($tosign);
    $this->authUnSigned="AWS {$this->accessKeyId}:".$tosign;
    $req =& new HTTP_Request($this->accessUrl);
    $req->addHeader("Date", $httpDate);
    $req->addHeader("Authorization", $authSigned);
    $req->sendRequest();
    $this->responseStr=$req->getResponseBody();
    $this->responseNice=$this->errorTranslate($req->getResponseCode());
    $this->responseHeaders=$req->_response->_headers;
    if ($this->responseInt != 200) {
      $this->_parseErrorStr();
      return(FALSE);
    } else {
      return($this->_parseBucketList());
    }    
  }


  /**
   * Retrieve a list of all files from a bucket (directory)
   *
   * Takes ($s3bucket)
   *
   * - [str] $s3bucket: the bucket/directory which you would like a listing of
   */
  function lsFiles($s3bucket, $pageKey=FALSE, $maxKeys=500) {
    $httpDate = gmdate(DATE_RFC2822);
    $tosign="GET\n\n\n{$httpDate}\n/".$s3bucket;
    $authSigned="AWS {$this->accessKeyId}:".$this->strSign($tosign);
    $this->authUnSigned="AWS {$this->accessKeyId}:".$tosign;
    if ( $pageKey ) {
      $req =& new HTTP_Request($this->accessUrl.$s3bucket."?max-keys={$maxKeys}&marker={$pageKey}");
    } else {
      $req =& new HTTP_Request($this->accessUrl.$s3bucket."?max-keys={$maxKeys}");
    }
    $req->addHeader("Date", $httpDate);
    $req->addHeader("Authorization", $authSigned);
    $req->sendRequest();
    $this->responseStr=$req->getResponseBody();
    $this->responseNice=$this->errorTranslate($req->getResponseCode());
    $this->responseHeaders=$req->_response->_headers;
    if ($this->responseInt != 200) {
      $this->_parseErrorStr();
      return(FALSE);
    } else {
      $rval=$this->_parseFileList();
      $is_more=array_shift($rval);
      if ( $is_more == "true" ) {
        $nrval=$this->lsFiles($s3bucket, $rval[(count($rval)-1)], $maxKeys);
        if ( is_array($nrval) ) {
          return(array_merge($rval, $nrval));
        }
      } else {
        return($rval);
      }
    }
  }

  /**
   * Delete a file object from your s3 account S3
   *
   * Takes ($s3bucket, $s3file)
   *
   * - [str] $s3bucket: the bucket/directory from which you would like the file
   * - [str] $s3file: the file which you would like from the bucket/directory
   */
  function rmFile($s3bucket, $s3file) {
    $httpDate = gmdate(DATE_RFC2822);
    $tosign="DELETE\n\n\n{$httpDate}\n/".$s3bucket.'/'.$s3file;
    $authSigned="AWS {$this->accessKeyId}:".$this->strSign($tosign);
    $this->authUnSigned="AWS {$this->accessKeyId}:".$tosign;
    $req =& new HTTP_Request($this->accessUrl.$s3bucket.'/'.$s3file);
    $req->setMethod("DELETE");
    $req->addHeader("Date", $httpDate);
    $req->addHeader("Authorization", $authSigned);
    $req->sendRequest();
    $this->responseStr=$req->getResponseBody();
    $this->responseNice=$this->errorTranslate($req->getResponseCode());
    $this->responseHeaders=$req->_response->_headers;
    // The response for a sucessful delete is, apparently, 204 :D
    if ($this->responseInt != 204) {
      $this->_parseErrorStr();
      return(FALSE);
    } else {
      return(TRUE);
    }
  }

  /**
   * Delete an empty s3 bucket/directory. (The operation will fail if you attempt this on a bucket which is not empty)
   *
   * Takes ($s3bucket, $recursive)
   *
   * - [str] $s3bucket: the bucket/directory which you would like to delete from s3.
   * - [int] $recursive: OPTIONAL: DEFAULTS: 0, if set to 1 it will delete all files inside the bucket, and then delete the bucket.  This is because attempting to delete a bucket which is not empty gives an error 409 "BucketNotEmpty" ("The bucket you tried to delete is not empty")
   */
  function rmBucket($s3bucket, $recursive=0) {
    if ( ! is_array($this->lsFiles($s3bucket)) ) {
      return(TRUE);
    }
    if ( $recursive == 1 ) {
      while( count($this->lsFiles($s3bucket)) > 0 ) {
        $files=$this->lsFiles($s3bucket);
        foreach ( $files as $val ) {
         $this->rmFile($s3bucket, $val);
        }
      }
    }
    $httpDate = gmdate(DATE_RFC2822);
    $tosign="DELETE\n\n\n{$httpDate}\n/".$s3bucket;
    $authSigned="AWS {$this->accessKeyId}:".$this->strSign($tosign);
    $authUnSigned="AWS {$this->accessKeyId}:".$tosign;
    $req =& new HTTP_Request($this->accessUrl.$s3bucket);
    $req->setMethod("DELETE");
    $req->addHeader("Date", $httpDate);
    $req->addHeader("Authorization", $authSigned);
    $req->sendRequest();
    $this->responseStr=$req->getResponseBody();
    $this->responseNice=$this->errorTranslate($req->getResponseCode());
    $this->responseHeaders=$req->_response->_headers;
    if ($this->responseInt != 204) {
      $this->_parseErrorStr();
      return(FALSE);
    } else {
      return(TRUE);
    }
  }

  /**
   * Save data from s3 into a local file
   *
   * Takes ($s3bucket, $s3file, $fileName)
   *
   * - [str] $s3bucket: The bucket/directory which holds the file
   * - [str] $s3file: The file, from, the bucket/directory which to retrieve
   * - [str] $fileName: The local filename in which to save your data
   */
  function saveFile($s3bucket, $s3file, $fileName) {
    $data=$this->getFile($s3bucket, $s3file);
    if ( $data ) {
      return(file_put_contents($fileName, $data));
    } else {
      return(FALSE);
    }
  }

  /**
   * Get data from an S3 file object and return it as a string
   *
   * Takes ($s3bucket, $s3file)
   *
   * - [str] $s3bucket: The bucket/directory which holds the file
   * - [str] $s3file: The file, from, the bucket/directory which to retrieve
   */
  function getFile($s3bucket, $s3file) {
    $httpDate = gmdate(DATE_RFC2822);
    $tosign="GET\n\n\n{$httpDate}\n/".$s3bucket.'/'.$s3file;
    $authSigned="AWS {$this->accessKeyId}:".$this->strSign($tosign);
    $authUnSigned="AWS {$this->accessKeyId}:".$tosign;
    $req =& new HTTP_Request($this->accessUrl.$s3bucket.'/'.$s3file);
    $req->addHeader("Date", $httpDate);
    $req->addHeader("Authorization", $authSigned);
    $req->sendRequest();
    $this->responseStr=$req->getResponseBody();
    $this->responseNice=$this->errorTranslate($req->getResponseCode());
    $this->responseHeaders=$req->_response->_headers;
    if ($this->responseInt != 200) {
      $this->_parseErrorStr();
      return(FALSE);
    } else {
      return($this->responseStr);
    }    
  }

  /**
   * Make an Amazon S3 container object (directory)
   *
   * Takes ($s3bucket)
   *
   * - [str] $s3bucket: the bucket/directory you wish to create
   */
  function mkBucket($s3bucket) {
    $content_md5='';
    $content_type='';
    $httpDate = gmdate(DATE_RFC2822);
    $tosign="PUT\n\n\n{$httpDate}\n/${s3bucket}";   
    $authSigned="AWS {$this->accessKeyId}:".$this->strSign($tosign);
    $authUnSigned="AWS {$this->accessKeyId}:".$tosign;
    $authPlain="AWS {$this->accessKeyId}:".$tosign;
    $req =& new HTTP_Request($this->accessUrl . $s3bucket);
    $req->setMethod("PUT");
    $req->addHeader("Content-Length", 0);
    $req->addHeader("Date", $httpDate);
    $req->addHeader("Authorization", $authSigned);
    $req->sendRequest();
    $this->responseStr=$req->getResponseBody();
    $this->responseNice=$this->errorTranslate($req->getResponseCode());
    $this->responseHeaders=$req->_response->_headers;
    if ( ($this->responseInt != 200) ) {
      $this->_parseErrorStr();
      return(FALSE);
    } else {
      return(TRUE);
    }    
  }

  /**
   * Read a file from the filesystem and put it into S3
   *
   * Takes ($string, $s3bucket, $s3file, $contentType, $contentExpires, $contentDisposition, $cacheControl)
   *
   * - [str] $fileName: The file contents you wish to write to the object file
   * - [str] $s3bucket: the bucket/directory you wish to store this file in
   * - [str] $s3file: the name of the file object in which you would like to store data in side the bucket/directory
   * - [str] $contentType: OPTIONAL: defaults: binary/octet-stream; see http://s3.amazonaws.com/doc/s3-developer-guide/RESTObjectPUT.html
   * - [str] $contentExpires: OPTIONAL: defaults: OFF, the unix timestamp at which the file expires, see http://s3.amazonaws.com/doc/s3-developer-guide/RESTObjectPUT.html
   * - [str] $contentDisposition: OPTIONAL: defaults: OFF, see http://s3.amazonaws.com/doc/s3-developer-guide/RESTObjectPUT.html
   * - [str] $cacheControl: OPTIONAL: defaults: OFF, see http://s3.amazonaws.com/doc/s3-developer-guide/RESTObjectPUT.html
   */
  function putFile($fileName, $s3bucket, $s3file, $contentType="binary/octet-stream", $contentExpires=FALSE, $contentDisposition=FALSE, $cacheControl=FALSE) {
    $content=file_get_contents($fileName);
    return($this->mkFile($content, $s3bucket, $s3file, $contentType, $contentExpires, $contentDisposition, $cacheControl));
  }

  /**
   * Put data (in the form of a passed string) into an S3 file object.  If you specify an object to which you have write privileges and already exists... this will overwrite that object
   *
   * Takes ($string, $s3bucket, $s3file, $contentType, $contentExpires, $contentDisposition, $cacheControl)
   *
   * - [str] $string: The file contents you wish to write to the object file
   * - [str] $s3bucket: the bucket/directory you wish to store this file in
   * - [str] $s3file: the name of the file object in which you would like to store data in side the bucket/directory
   * - [str] $contentType: OPTIONAL: defaults: binary/octet-stream; see http://s3.amazonaws.com/doc/s3-developer-guide/RESTObjectPUT.html
   * - [str] $contentExpires: OPTIONAL: defaults: OFF, the unix timestamp at which the file expires, see http://s3.amazonaws.com/doc/s3-developer-guide/RESTObjectPUT.html
   * - [str] $contentDisposition: OPTIONAL: defaults: OFF, see http://s3.amazonaws.com/doc/s3-developer-guide/RESTObjectPUT.html
   * - [str] $cacheControl: OPTIONAL: defaults: OFF, see http://s3.amazonaws.com/doc/s3-developer-guide/RESTObjectPUT.html
   */
  function mkFile($string, $s3bucket, $s3file, $contentType="binary/octet-stream", $contentExpires=FALSE, $contentDisposition=FALSE, $cacheControl=FALSE, $defaultAcl=FALSE) {
    $req =& new HTTP_Request($this->accessUrl . $s3bucket .'/'. $s3file);
    $req->setMethod("PUT");
    $httpDate = gmdate(DATE_RFC2822);
    $contentLength=strlen($string);

    // Mandatory headers
    $tosign[]="PUT";
    // TODO: Impliment md5 checksums
    $tosign[]='';
    // $contentMd5=md5($string, TRUE);
    // $contentMd5=$this->md5_base64($string);
    // $tosign[]=$contentMd5;
    // $req->addHeader("Content-MD5", $contentMd5);
    if ( $contentType ) { 
      $tosign[]=$contentType;    
      $req->addHeader("Content-Type", $contentType); 
    } else { 
      $tosign[]="";  
    }
    $tosign[]=$httpDate;
    $tosign[]='/'.$s3bucket.'/'.$s3file;
    $req->addHeader("Date", $httpDate);
    $req->addHeader("Content-Length", $contentLength);
    $this->authUnSigned=urlencode("AWS {$this->accessKeyId}:".implode("\n", $tosign));
    $authSigned="AWS {$this->accessKeyId}:".$this->strSign(implode("\n", $tosign));
    $req->addHeader("Authorization", $authSigned);

    // Optional Headers
    if ( $contentExpires ) {
      $req->addHeader("Expires", $contentExpires);      
    } 
    if ( $contentDisposition ) { 
      $req->addHeader("Content-Disposition", $contentDisposition);      
    } 
    if ( $cacheControl ) { 
      $req->addHeader("Cache-Control", $cacheControl); 
    }    

    // TODO: Impliment arbitrary headers via associative array

    $req->setBody($string);
    $req->sendRequest();

    $this->responseStr=$req->getResponseBody();
    $this->responseNice=$this->errorTranslate($req->getResponseCode());
    $this->responseHeaders=$req->_response->_headers;
    if ($this->responseInt != 200) {
      $this->_parseErrorStr();
      return(FALSE); 
    } else {
      if ( $defaultAcl ) {
        return($this->setACL($s3bucket, $s3file, $shorthand=$defaultAcl));
      } else {
        return(TRUE);
      }
    }    
  }


  /**
   *  Find out if an object already exists in s3
   *
   *  - Takes ($s3bucket, $s3file)
   *  - Returns bool (TRUE: exists, FALSE: doesn't exist)
   *
   *  - [str] $s3bucket, REQUIRED, the bucket in which the target object is located
   *  - [str] $s3file, REQUIRED, the object on which you wish to check
   */
  function fileExists($s3bucket, $s3file) {
    $req =& new HTTP_Request($this->accessUrl . $s3bucket .'/'. $s3file);
    $req->setMethod("HEAD");
    $httpDate = gmdate(DATE_RFC2822);
    $tosign[]="HEAD";
    $tosign[]='';
    $tosign[]='';  
    $tosign[]=$httpDate;
    $tosign[]='/'.$s3bucket.'/'.$s3file;
    $req->addHeader("Date", $httpDate);
    $this->authUnSigned="AWS {$this->accessKeyId}:".implode("\n", $tosign);
    $authSigned="AWS {$this->accessKeyId}:".$this->strSign(implode("\n", $tosign));
    $req->addHeader("Authorization", $authSigned);
    $req->setBody($string);
    $req->sendRequest();
    $this->responseStr=$req->getResponseBody();
    $this->responseNice=$this->errorTranslate($req->getResponseCode());
    $this->responseHeaders=$req->_response->_headers;
    if ($this->responseInt != 200) {
      $this->_parseErrorStr();
      return(FALSE); 
    } else {
      return(TRUE);
    }    
    
  }

  /**
   *  Set the acl for a file stored in S3
   *
   *  - Takes ($s3bucket, $s3file, $shorthand)
   *  - Returns bool TRUE|FALSE
   *
   *  - [str] $s3bucket, REQUIRED, the bucket in which the target object is located
   *  - [str] $s3file, REQUIRED, the object on which you wish to set the ACL
   *  - [str] $shorthand, OPTIONAL, DEFAULT: "public-read"
   *
   *  Contributed by "Nick" (http://www.katanapg.com/) April 5th, 2006 at 9:52 pm e
   */
  function setACL($s3bucket, $s3file, $shorthand="public-read") {
    $s3file = $s3file . "?acl";
    $req =& new HTTP_Request($this->accessUrl . $s3bucket ."/". $s3file);
    //echo $this->accessUrl . $s3bucket ./. $s3file;
    $req->setMethod("PUT");
    $httpDate = gmdate(DATE_RFC2822);
    // Mandatory headers
    $tosign[]="PUT";
    // TODO: Impliment md5 checksums
    $tosign[]="";
    $tosign[]="";
    $tosign[]=$httpDate;
    $tosign[]="x-amz-acl:" . $shorthand;
    $tosign[]="/".$s3bucket."/".$s3file;
    $req->addHeader("Date", $httpDate);
    $this->authUnSigned="AWS {$this->accessKeyId}:".implode("\n", $tosign);
    $authSigned="AWS {$this->accessKeyId}:".$this->strSign(implode("\n", $tosign));
    $req->addHeader("Authorization", $authSigned);
    $req->addHeader("x-amz-acl", $shorthand);
    $req->sendRequest();
    $this->responseStr=$req->getResponseBody();
    $this->responseNice=$this->errorTranslate($req->getResponseCode());
    if ($this->responseInt != 200) {
      $this->_parseErrorStr();
      return(FALSE);
    } else {
      return(TRUE);
    }
  } 



  /**
   * Take the XMl returned from amazon s3 and make an array out of it
   *
   * @ignore
   */
  function _parseBucketList() {
    $parser=simplexml_load_string($this->responseStr);
    $rval=array();
    foreach ( $parser->Buckets->Bucket as $bucket ) {
      list(, $dir) = each($bucket);
      $rval[]=$dir;
    }
    return($rval);
  }

  /**
   * Take the XMl returned from amazon s3 and make an array out of it
   *
   * @ignore
   */
  function _parseFileList() {
    $parser=simplexml_load_string($this->responseStr);
    $rval=array();
    list(, $IsTruncated)=each($parser->IsTruncated);
    $rval["IsTruncated"]=$IsTruncated;
    foreach ( $parser->Contents as $item ) {
      list(, $file)=each($item->Key);
      $rval[]=$file; 
    }
    return($rval);
  }

  /**
   * Take the XMl returned from amazon s3 and make an array out of it
   *
   * @ignore
   */
  function _parseErrorStr() {
    $parser=simplexml_load_string($this->responseStr);
    if ( $parser ) {
      foreach ( $parser as $line => $val ) {
        list(, $value)=each($val);
        $rval[$line]=$value; 
      }
    }
    if ( isset($rval) ) {
      $this->responseArray=$rval;
    } else {
      return(TRUE);
    }
  }

  /**
   * Convert a hex string to a base64 string (stolen from the example)
   *
   * @IGNORE
   */
  function hex2b64($str) {
      $raw = '';
      for ($i=0; $i < strlen($str); $i+=2) {
          $raw .= chr(hexdec(substr($str, $i, 2)));
      }
      return base64_encode($raw);
  }

  /**
   * Sign a string, as directed by the s3 api (stolen from the example)
   *
   * @ignore
   */
  function strSign($str) {
    $hasher =& new Crypt_HMAC($this->accessSecret, "sha1");
    $signature = $this->hex2b64($hasher->hash($str));
    return($signature);
  }

  /**
   * May be used later on to determine how the class behaves when encountering particular response codes.
   *
   * @ignore
   */
  function errorTranslate($int) {
   $this->responseInt=$int; 
  }

  /**
   * Tell me why something failed!
   *
   * Returns an array of the values returned, via the xml output, from the s3 service
   */
  function why() {
    if ( $this->responseInt != 200 ) {
      $rval=$this->responseArray;
      $rval['int']=$this->responseInt;
      return($rval);
    } else {
      $rval['int']=$this->responseInt;
      return($rval);
    }
  }

}
?>
